﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Collections;
using System.Windows.Threading;

namespace DataPresenterExternalOperations
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
        IList dataSource;

        public MainWindow( )
		{
			InitializeComponent( );
            dataSource = CreateDataSource( 100000 );
            _100kInternalGrid.DataSource = new ListCollectionView(dataSource);
            _100kExternalGrid.DataSource = new ListCollectionView(dataSource);
		}

		private IList CreateDataSource( int itemCount )
		{
			BindingList<Item> items = new BindingList<Item>( );
			const int COUNT = 100000;

			Random r = new Random( );
			DateTime now = DateTime.Now;

			for ( int i = 0; i < COUNT; i++ )
			{
				Item item = new Item
				{
					A = "a" + r.Next( 100 ),
					B = "b" + r.Next( 100 ),
					C = "c" + r.Next( 100 ),
					D = "d" + r.Next( 100 ),
					DateField = now.AddDays( r.Next( 100 ) ).Date,
					IntField = r.Next( 200 ),
					FloatField = (float)( r.NextDouble( ) * r.Next( 1000 ) ),
					DoubleField = r.NextDouble( ) * r.Next( 1000 ),
					DecimalField = (decimal)( r.NextDouble( ) * r.Next( 1000 ) ),
				};

				items.Add( item );
			}

            return items;
		}

		private void _100kInternalGrid_Sorting( object sender, Infragistics.Windows.DataPresenter.Events.SortingEventArgs e )
		{
			_internalTimeLabel.Content = "Sorting...";
            ReportTimeAndMemory(_internalTimeLabel, "sort");
		}

		private void _100kExternalGrid_Sorting( object sender, Infragistics.Windows.DataPresenter.Events.SortingEventArgs e )
		{
			_externalTimeLabel.Content = "Sorting...";
			ReportTimeAndMemory(_externalTimeLabel, "sort");
		}

		private void _100kInternalGrid_Grouping( object sender, Infragistics.Windows.DataPresenter.Events.GroupingEventArgs e )
		{
			_internalTimeLabel.Content = "Grouping...";
            ReportTimeAndMemory(_internalTimeLabel, "group");
		}

		private void _100kExternalGrid_Grouping( object sender, Infragistics.Windows.DataPresenter.Events.GroupingEventArgs e )
		{
			_externalTimeLabel.Content = "Grouping...";
            ReportTimeAndMemory(_externalTimeLabel, "group");
		}

        private void ReportTimeAndMemory(Label label, string operation)
        {
            DateTime start = DateTime.Now;
            long currentMemUsage = GC.GetTotalMemory(true);
            this.Dispatcher.BeginInvoke(DispatcherPriority.Background,
                new Action(() =>
                {
                    label.Content = "Time to " + operation + "= " + (DateTime.Now - start).TotalSeconds + " Memory usage delta: " + (GC.GetTotalMemory(true) - currentMemUsage) / (1024 * 1024) + " MB"; ;
                })
            );
        }

	}
}
