﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Windows.DataPresenter;
using System.Windows.Media.Effects;
using System.Diagnostics;
using System.Windows.Threading;

namespace XamDataGridTickingPerformance
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        ListCollectionView lcv;
        int largeDatasetCount = 2000;
        TickerBuffer tickerBuffer;
        ResourceDictionary readonlyStyle = new ResourceDictionary() { Source = new Uri("ReadOnlyHighPerformanceStyle.xaml", UriKind.Relative) };
        ResourceDictionary gridleanStyle = new ResourceDictionary() { Source = new Uri("ReadOnly_Gridlines.xaml", UriKind.Relative) };
        ResourceDictionary gridleanStyleConditionallyFormatted = new ResourceDictionary() { Source = new Uri("GridLeanStylesConditionallyFormatted.xaml", UriKind.Relative) };

        public MainWindow()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(Window_Loaded);
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (Debugger.IsAttached)
                MessageBox.Show(@"You're running the demo in debug mode which will overestimate CPU load values. 
Please run the demo without debugger attached to produce CPU load values you will experience in the production environment.",
                    "XamDataGrid Performance Demo",
                    MessageBoxButton.OK,
                    MessageBoxImage.Exclamation);

            //MessageBox.Show("Sorting through the ListCollectionView is updated only when INotifyCollectionChanged events are raised.");

            tickerBuffer = new TickerBuffer(largeDatasetCount);
            tickerBuffer.WindowSize = largeDatasetCount;
            //tickerBuffer._dispatcherProvider = this;

            //wrapping the data in a list collection view with an applied sort
            lcv = new ListCollectionView(tickerBuffer);

            xamDataGrid1.DataSource = lcv;
            //add a column to pre-sort the data
            //lcv.SortDescriptions.Add(new System.ComponentModel.SortDescription("CA", System.ComponentModel.ListSortDirection.Ascending));

            //add a style to the pre-sorted column to indicate this is the sorted column
            //xamDataGrid1.FieldLayouts[0].Fields["CA"].Settings.CellValuePresenterStyle = (Style)this.Resources["columnACellStyle"];
        }

        private void collectionEvents_Clicked(object sender, RoutedEventArgs e)
        {
            TickerBuffer.IsCollectionChangedActivated = true;
            TickerBuffer.IsPropertyThrottleActivated = false;
            Order.IsPropertyChangedActivated = false;

            //no need to resort data source after updates, this is done automatically when the data source is sent a Reset change notification
        }

        private void propertyEvents_Clicked(object sender, RoutedEventArgs e)
        {
            Order.IsPropertyChangedActivated = true;
            TickerBuffer.IsPropertyThrottleActivated = false;
            TickerBuffer.IsCollectionChangedActivated = false;

            //in the initialize record direct we will resort all updated rows
        }

        private void propertyThrottleChk_Click(object sender, RoutedEventArgs e)
        {
            TickerBuffer.IsPropertyThrottleActivated = true;
            TickerBuffer.IsCollectionChangedActivated = false;
            Order.IsPropertyChangedActivated = false;
        }

        private void xamDataGrid1_InitializeRecordDirect(object sender, Infragistics.Windows.DataPresenter.Events.InitializeRecordEventArgs e)
        {
            //we only need to refresh sort position on records when not using Reset notifications on the bound data source
            if (!TickerBuffer.IsCollectionChangedActivated)
            {
                if (e.SortValueChanged)
                    e.Record.RefreshSortPosition();
            }
        }


        private void feedSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            //if (tickerBuffer != null)
            //    tickerBuffer.FeedUpdateInterval = TimeSpan.FromMilliseconds((sender as Slider).Value);
        }


        DateTime _timingStart;

        //private void _scrollButton_Click(object sender, RoutedEventArgs e)
        //{
        //    _timingStart = DateTime.Now;

        //    int elementsNumberToScroll = Int32.Parse(_scrollBy.Text);
        //    int timesToScroll = Int32.Parse(_scrollTimes.Text);
        //    int currentPosition = (int)xamDataGrid1.ScrollInfo.ScrollOwner.VerticalOffset;

        //    for (int i = 1; i <= timesToScroll; i++)
        //        xamDataGrid1.ScrollInfo.ScrollOwner.ScrollToVerticalOffset(currentPosition + elementsNumberToScroll * timesToScroll);
        //}

        private void xamDataGrid1_RecordsInViewChanged(object sender, Infragistics.Windows.DataPresenter.Events.RecordsInViewChangedEventArgs e)
        {
            if (_timingStart == DateTime.MinValue)
                return;
            _scrollingResult.Text = DateTime.Now.Subtract(_timingStart).TotalMilliseconds.ToString();
            _timingStart = DateTime.MinValue;
        }

        private void readonlyStyleRadio_Checked(object sender, RoutedEventArgs e)
        {
            if (xamDataGrid1 == null)
                return;

            xamDataGrid1.Resources.MergedDictionaries.Clear();
            xamDataGrid1.Resources.MergedDictionaries.Add(readonlyStyle);
        }

        private void normalStyleRadio_Checked(object sender, RoutedEventArgs e)
        {
            if (xamDataGrid1 == null)
                return;

            xamDataGrid1.Resources.Clear();
            xamDataGrid1.Resources.MergedDictionaries.Clear();
        }

        private void gridleanStyleRadio_Checked(object sender, RoutedEventArgs e)
        {
            if (xamDataGrid1 == null)
                return;

            xamDataGrid1.Resources.MergedDictionaries.Clear();
            xamDataGrid1.Resources.MergedDictionaries.Add(gridleanStyle);
        }

        private void gridleanStyleCondRadio_Checked(object sender, RoutedEventArgs e)
        {
            if (xamDataGrid1 == null)
                return;

            xamDataGrid1.Resources.MergedDictionaries.Clear();
            xamDataGrid1.Resources.MergedDictionaries.Add(gridleanStyleConditionallyFormatted);
        }

        private void _expandButton_Click(object sender, RoutedEventArgs e)
        {
            xamDataGrid1.Records.ExpandAll(true);
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            _start = DateTime.Now;
            _lastVerticalOffset = -1;
            this.xamDataGrid1.ScrollInfo.SetVerticalOffset(0.0);
            this.BenchmarkScrollingAsync();
        }

        private double _lastVerticalOffset;
        private DateTime _start;

        private void BenchmarkScrollingAsync()
        {
            this.Dispatcher.BeginInvoke(DispatcherPriority.Input, new Action(this.BenchmarkScrolling));
        }

        private void BenchmarkScrolling()
        {
            double offset = this.xamDataGrid1.ScrollInfo.VerticalOffset;

            if (_lastVerticalOffset == offset)
            {
                _scrollingResult.Text = "Time = " + (DateTime.Now - _start).TotalSeconds;
                _lastVerticalOffset = -1;
            }
            else
            {
                _lastVerticalOffset = offset;
                this.xamDataGrid1.ScrollInfo.PageDown();
                this.BenchmarkScrollingAsync();
            }
        }
    }
}
