﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Windows.DataPresenter;
using System.Windows.Media.Effects;
using System.Diagnostics;
using System.ComponentModel;

namespace XamDataGridPerformance
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public CountsVM countsVM = new CountsVM();

        public MainWindow()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(Window_Loaded);
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.DataContext = countsVM;
            xamDataGrid1.Records.ExpandAll(true);
        }

        private void xamDataGrid1_InitializeRecordDirect(object sender, Infragistics.Windows.DataPresenter.Events.InitializeRecordEventArgs e)
        {
            //we have to call the RefreshSortPosition method, or else the records do not get their group placement updated after a data update
            if (e.ReInitialize)
            {
                ++countsVM.ReinitializeCount;
                //old way of refreshing record position - no way to find out whether it's the sorted property that changed, or any other property of the record
                //this is a wasteful method, which invokes the repositioning logic much more frequently than actually needed
                
                //e.Record.RefreshSortPosition();
            }

            if (e.SortValueChanged)
            {
                ++countsVM.SortValueChangedCount;
                //new way of refreshing record position - the SortValueChanged flag is only true when the sorted/grouped property changes
                //this is the optimal way of finding out whether we need to call the RefreshSortPosition, only invoking the repositioning logic when actually needed
                e.Record.RefreshSortPosition();
            }
        }
    }

    public class CountsVM : INotifyPropertyChanged
    {
        int _reinitCount = 0;
        int sortChangedCount = 0;
        public int ReinitializeCount
        {
            get
            {
                return _reinitCount;
            }
            set
            {
                _reinitCount = value;
                RaisePropertyChanged("ReinitializeCount");
            }
        }
        public int SortValueChangedCount
        {
            get
            {
                return sortChangedCount;
            }
            set
            {
                sortChangedCount = value;
                RaisePropertyChanged("SortValueChangedCount");
            }
        }

        public void RaisePropertyChanged(string name)
        {
            if(PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
